==> copilot-24.7.3-sources.jar: com/vaadin/copilot/ai/AICommunicationUtil.java <==
package com.vaadin.copilot.ai;

import java.util.LinkedHashMap;
import java.util.Map;
import java.util.concurrent.TimeoutException;

import com.vaadin.base.devserver.DevToolsInterface;
import com.vaadin.copilot.Copilot;
import com.vaadin.copilot.CopilotCommand;
import com.vaadin.copilot.communication.CopilotServerResponseCode;
import com.vaadin.copilot.userinfo.AIUsageDisabledException;

import elemental.json.Json;
import elemental.json.JsonArray;
import elemental.json.JsonObject;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class AICommunicationUtil {

    /**
     * Method handling the case when the AI cannot be called, because the proKey or
     * machineId is missing.
     *
     * @param data
     *            - the data object, we are getting from the client/browser
     * @param devToolsInterface
     *            - the devtools interface we use to communicate with the
     *            client/browser
     */
    public static void promptTextCannotCall(JsonObject data, DevToolsInterface devToolsInterface) {
        getLogger().error("Cannot call AI without a proKey or machineId present.");
        JsonObject responseData = Json.createObject();
        if (data.hasKey(CopilotCommand.KEY_REQ_ID)) {
            responseData.put(CopilotCommand.KEY_REQ_ID, data.getString(CopilotCommand.KEY_REQ_ID));
        }
        responseData.put("error", true);
        responseData.put("code", CopilotServerResponseCode.COPILOT_AUTHORISATION_FAILED.getCode());
        devToolsInterface.send(Copilot.PREFIX + AIConstants.MESSAGE_PROMPT_FAILED, responseData);
    }

    public static Map<String, String> getHillaSourceFiles(JsonObject data) {
        Map<String, String> sources = new LinkedHashMap<>();

        // Hilla source files;
        JsonArray reactSources = data.getArray("sources");
        if (reactSources != null) {
            for (int i = 0; i < reactSources.length(); i++) {
                JsonObject source = reactSources.getObject(i);
                getLogger().debug("Hilla file: {}", source.getString("filename"));
                sources.put(source.getString("filename"), source.getString("projectFile"));
            }
        }
        return sources;
    }

    public static void handlingExceptionsAndNotifyDevtoolsInterface(JsonObject dataJson,
            DevToolsInterface devToolsInterface, Throwable e) {
        JsonObject responseData = Json.createObject();
        if (dataJson.hasKey(CopilotCommand.KEY_REQ_ID)) {
            responseData.put(CopilotCommand.KEY_REQ_ID, dataJson.getString(CopilotCommand.KEY_REQ_ID));
        }
        if (e.getCause() instanceof TimeoutException) {
            responseData.put("code", AIConstants.COPILOT_CLIENT_TIMEOUT_ERROR);
        } else if (e instanceof AIUsageDisabledException) {
            responseData.put("code", CopilotServerResponseCode.AI_DISABLED.getCode());
        } else {
            responseData.put("code", AIConstants.COPILOT_INTERNAL_ERROR);
        }
        responseData.put("message", "Error querying copilot server");
        getLogger().error("Error querying copilot server", e);
        devToolsInterface.send(Copilot.PREFIX + AIConstants.MESSAGE_PROMPT_FAILED, responseData);
    }

    private static Logger getLogger() {
        return LoggerFactory.getLogger(AICommunicationUtil.class);
    }
}

==> copilot-24.8.9-sources.jar: com/vaadin/copilot/ai/AICommunicationUtil.java <==
package com.vaadin.copilot.ai;

import java.util.LinkedHashMap;
import java.util.Map;
import java.util.concurrent.TimeoutException;

import com.vaadin.base.devserver.DevToolsInterface;
import com.vaadin.copilot.Copilot;
import com.vaadin.copilot.CopilotCommand;
import com.vaadin.copilot.communication.CopilotServerResponseCode;
import com.vaadin.copilot.userinfo.AIUsageDisabledException;
import com.vaadin.flow.internal.JacksonUtils;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ObjectNode;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class AICommunicationUtil {

    /**
     * Method handling the case when the AI cannot be called, because the proKey or
     * machineId is missing.
     *
     * @param data
     *            - the data object, we are getting from the client/browser
     * @param devToolsInterface
     *            - the devtools interface we use to communicate with the
     *            client/browser
     */
    public static void promptTextCannotCall(JsonNode data, DevToolsInterface devToolsInterface) {
        getLogger().error("Cannot call AI without a proKey or machineId present.");
        ObjectNode responseData = JacksonUtils.createObjectNode();
        if (data.has(CopilotCommand.KEY_REQ_ID)) {
            responseData.put(CopilotCommand.KEY_REQ_ID, data.get(CopilotCommand.KEY_REQ_ID).asText());
        }
        responseData.put("error", true);
        responseData.put("code", CopilotServerResponseCode.COPILOT_AUTHORISATION_FAILED.getCode());
        devToolsInterface.send(Copilot.PREFIX + AIConstants.MESSAGE_PROMPT_FAILED, responseData);
    }

    public static Map<String, String> getHillaSourceFiles(JsonNode data) {
        Map<String, String> sources = new LinkedHashMap<>();

        // Hilla source files;
        if (data.hasNonNull("sources")) {
            for (JsonNode source : data.withArray("sources")) {
                getLogger().debug("Hilla file: {}", source.get("filename").asText());
                sources.put(source.get("filename").asText(), source.get("projectFile").asText());
            }
        }
        return sources;
    }

    public static void handlingExceptionsAndNotifyDevtoolsInterface(JsonNode dataJson,
            DevToolsInterface devToolsInterface, Throwable e) {
        ObjectNode responseData = JacksonUtils.createObjectNode();
        if (dataJson.has(CopilotCommand.KEY_REQ_ID)) {
            responseData.put(CopilotCommand.KEY_REQ_ID, dataJson.get(CopilotCommand.KEY_REQ_ID).asText());
        }
        if (e.getCause() instanceof TimeoutException) {
            responseData.put("code", AIConstants.COPILOT_CLIENT_TIMEOUT_ERROR);
        } else if (e instanceof AIUsageDisabledException) {
            responseData.put("code", CopilotServerResponseCode.AI_DISABLED.getCode());
        } else {
            responseData.put("code", AIConstants.COPILOT_INTERNAL_ERROR);
        }
        responseData.put("message", "Error querying copilot server");
        getLogger().error("Error querying copilot server", e);
        devToolsInterface.send(Copilot.PREFIX + AIConstants.MESSAGE_PROMPT_FAILED, responseData);
    }

    private static Logger getLogger() {
        return LoggerFactory.getLogger(AICommunicationUtil.class);
    }
}
